#ifndef LIBDEFINE_H
#define LIBDEFINE_H

#include <stdio.h>
#include "LibFileInfo.h"
#include "LibParse.h"
#include "LibVal.h"
#include "UHash"

class LibGroup;

class LibDefine : public UEHashNode
{
public:

  LibDefine() : d_valType(LIBERTY__VAL_UNDEFINED), d_owner(NULL) {}
  LibDefine(const UString& nm, const UString& allowedGrp, LibValType vtype,
	    LibGroup* parent, const LibFileInfo& f) :
    d_name(nm), d_groupType(allowedGrp), d_valType(vtype),
    d_owner(parent), d_info(f) {
  }

  void               groupType(const UString& gt)   { d_groupType = gt; }
  void               fileInfo(const LibFileInfo& f) { d_info = f; }

  const UString&     name()                   const { return d_name; }
  const UString&     groupType()              const { return d_groupType; }
  LibValType         valType()                const { return d_valType; }
  int                lineno()                 const { return d_info.lineno(); }
  const char*        filename()               const { return LibParse::filename();}
  LibGroup*          owner()                  const { return d_owner; }
  void               dump(FILE* f, const char* indent);

private:

  UString            d_name;
  UString            d_groupType;
  LibValType         d_valType;
  LibGroup*          d_owner;
  LibFileInfo        d_info;

};

inline void
LibDefine::dump(FILE* f, const char* indent)
{
  const char *vt;
  
  switch (d_valType) {
  case LIBERTY__VAL_STRING:
    vt = ",string";
    break;
  case LIBERTY__VAL_DOUBLE:
    vt = ",float";
    break;
  case LIBERTY__VAL_INT:
    vt = ",integer";
    break;
  default:
    vt = "";
    break;
  }
  fprintf(f, "%sdefine(%s,%s%s);\n", indent, (str_t)d_name,
	  (str_t)d_groupType, vt);
}

template <> struct UEKey<LibDefine*>
{
  const UString& operator()(LibDefine* def) const {
    return def->name();
  }
};

#endif

